/************************************************************
File name: 0_DEFINES.vh
************************************************************/
`define n (32)      // internal word
`define p (16)      // number of cells
`define m (1024)    // memory size

                       //////////
                       // HOST //
                       //////////

/************************************************************
Name: Host's instruction set architecture

instr={opCode[5:0],dest[4:0],left[4:0],right[4:0],nu[10:0]} |
      {opCode[5:0],dest[4:0],left[4:0],val[15:0]}
reg [31:0]              rf[0:31]
reg [$clog2(`m)-1:0]    pc
reg                     cr
************************************************************/
// CONTROL INSTRUCTIONS
`define hnop     6'b000000 // pc <= pc+1
`define hjmp     6'b000001 // pc <= pc+val
`define hjmpz    6'b000010 // pc <= (rf[left]==0) ? pc+val:pc+1
`define hjmpnz   6'b000011 // pc <= (rf[left]==0) ? pc+1:pc+val
`define hpsend   6'b100000 // pc <= (host2int[31:24]==
                           // {`prun,`ctl}) ? pc + 1 : pc
                           // h2iPwrite = 1
                           // rf[left] <= rf[left]+!i2hPfull
`define hdget    6'b100010 // pc <= (leftOut==rightOut) ?
                           // pc + 1 : pc
                           // i2hDread = 1
                           // rf[left] <= rf[left]+!i2hDfull
`define hdsend   6'b100001 // pc <= (leftOut==rightOut) ?
                           // pc + 1 : pc
                           // h2iDwrite = 1
                           // rf[left] <= rf[left]+!i2hDfull
`define hintwait 6'b000111 // pc <= (int) ? pc+1 : pc
`define hajmp    6'b001000 // pc <= val
`define hhalt    6'b001001 // pc <= pc
`define hsttcc   6'b001010 // start host cycle counter
`define hstpcc   6'b001011 // stop host cycle counter

// FUNCTIONAL INSTRUCTIONS
`define hadd     6'b010000
    // {cr,rf[dest]} <= rf[left]+rf[right]
`define haddcr   6'b010001
    // {cr,rf[dest]} <= rf[left]+rf[right]+cr
`define hsub     6'b010010
    // {cr,rf[dest]} <= rf[left]-rf[right]
`define hsubcr   6'b010011
    // {cr,rf[dest]} <= rf[left]-rf[right]-cr
`define hmult    6'b010100
    // {cr,rf[dest]} <= {cr, rf[left]*rf[right]}
`define hbwand   6'b010101
    // {cr,rf[dest]} <= {cr, rf[left] & rf[right]}
`define hbwor    6'b010110
    // {cr,rf[dest]} <= {cr, rf[left] | rf[right]}
`define hbwxor   6'b010111
    // {cr,rf[dest]} <= {cr, rf[left] ^ rf[right]}
`define haddv    6'b011000
    // {cr,rf[dest]} <= rf[left]+{{16{val[15}},val}
`define haddcrv  6'b011001
    // {cr,rf[dest]} <= rf[left]+{{16{val[15}},val}+cr
`define hsubv    6'b011010
    // {cr,rf[dest]} <= rf[left]-{{16{val[15}},val}
`define hsubcrv  6'b011011
    // {cr,rf[dest]} <= rf[left]-{{16{val[15}},val}-cr
`define hmultv   6'b011100
    // {cr,rf[dest]} <= rf[left]*{{16{val[15}},val}
`define hbwandv  6'b011101
    // {cr,rf[dest]} <= {cr,rf[left]&{{16{val[15}},val}}
`define hbworv   6'b011110
    // {cr,rf[dest]} <= {cr,rf[left]|{{16{val[15}},val}}
`define hbwxorv  6'b011111
    // {cr,rf[dest]} <= {cr,rf[left]^{{16{val[15}},val}}

// DATA TRANSFER INSTRUCTIONS
`define hfsend   6'b100011 // function send
    // {code[5:0], `prun, `ctl, initAddress}
`define hssend   6'b100100 // scalar send
    // {code[5:0], scalar}
`define hvalue   6'b101000
    // {cr,rf[dest]} <= {cr, {{16{val[15]}},val}}
`define hinsval  6'b101001
    // {cr,rf[dest]} <= {cr, {rf[left][15:0],val}
`define hload    6'b101010
    // {cr,rf[dest]} <= {cr, hDmem[rf[left]}
`define hstore   6'b111011
    // hDmem[rf[left] <= rf[right]

                //////////////////////////////
                // ACCELERATOR'S CONTROLLER //
                //////////////////////////////

/************************************************************
 Accelerator's controller state:
************************************************************/
`define idle    2'b00
`define loading 2'b01
`define running 2'b10

/************************************************************
Name: Controller's instruction set architecture

instr = {cOpcodeBinary[4:0] , // operation code for binary operations
         cMode[2:0]         , // second operand selection mode
         value[23:0]        } // signed integer value
         |
        {8'b11111_000       , 
         cOpcodeUnary[4:0]  , // operation code for unary operations
         value[18:0]        } // integer to parameterize operations

reg [$clog2(`m)-1:0] pc         // program counter
reg [31:0]           acc        // controller's accumuulator
reg                  cr         // controller's carry
reg[$clog2(`m)-1:0]  addrReg    // controller's address reg.
reg[31:0]            mem[0:m-1] // controller's data memory

val = {{8{value[23]}},value}    // used as operand
addr = value[$clog2(`m)-1:0]    // used as address

coOp = redOut
************************************************************/

/************************************************************
 Addressing modes defining the right operand:
************************************************************/
`define	imm 3'b000 // op: val
`define dir 3'b001 // op: mem[addr]
`define rel	3'b010 // op: mem[addrReg+addr]
`define rei 3'b011 // op: mem[addrReg+addr]; addrReg = addrReg+addr
`define cim 3'b100 // op: coOp
`define cdr 3'b101 // op: mem[coOp]
`define	crl 3'b110 // op: mem[addrReg+coOp]
`define cri	3'b111 // op: mem[addrReg+coOp]; addrReg = addrReg+coOp	

// FUNCTIONAL INSTRUCTIONS
`define cadd   	5'b00000 // {cr,acc} <= acc + op
`define caddcr 	5'b00001 // {cr,acc} <= acc + op + cr
`define csub   	5'b00010 // {cr,acc} <= acc - op
`define csubcr  5'b00011 // {cr,acc} <= acc - op - cr
`define cmult   5'b00100 // {cr,acc} <= {cr, acc * op}
`define cbwand  5'b00101 // {cr,acc} <= {cr, acc & op}
`define cbwor   5'b00110 // {cr,acc} <= {cr, acc | op}
`define cbwxor  5'b00111 // {cr,acc} <= {cr, acc ^ op}
// DATA MOVE INSTRUCTIONS
`define cload   5'b01000 // {cr,acc} <= {cr, op}
`define cstore  5'b01001 // cMem <= acc
`define cinsval 5'b01010  // {cr,acc} <= {cr, (acc[23:0],val[7:0]}

// SELECTS NO-OP INSTRUCTIONS
`define contr   5'b11111 // instruction: instr[23:19]

// CONTROL & GLOBAL INSTRUCTIONS for instr[31:24]= 11111_000
`define nop     5'b00000 // pc <= pc+1
`define jmp    	5'b00001 // pc <= pc+addr
`define brz     5'b00010 // pc <= (acc = 0) ? pc+addr : pc+1
`define brnz    5'b00011 // pc <= (acc = 0) ? pc+1 : pc+addr
`define brzdec  5'b00100 // pc <= (acc = 0) ? pc+addr : pc+1
`define brnzdec	5'b00101 // pc <= (acc = 0) ? pc+1 : pc+addr
`define ajmp    5'b00110 // pc <= addr
`define halt    5'b00111 // pc <= pc

`define setint  5'b01000 // set interrupt

`define datains	5'b01001 // insert data in array
`define dataext 5'b01010 // extract data from arrray

`define param   5'b01011 // load parameter
`define pload   5'b01100 // load program starting to val
`define prun    5'b01101 // run the program at val

`define start   5'b01110 // start cycle counter
`define stop    5'b01111 // stop cycle coounter

//`define   redload 5'b11100 // acc <= redOut
`define gshift 	5'b11101 // global_shift(val[2:0])

`define crela   5'b11110 // cAddrReg <= acc; load address
`define cshift 	5'b11111 // local_shift(val[2:0])


                    /////////////////////////
                    // ACCELERATOR'S ARRAY //
                    /////////////////////////

/************************************************************
Name: Array's instruction set architecture

instr = {opcodeBinary[4:0]  , // operation code for binary operations
         mode[2:0]          , // second operand selection mode
         value[23:0]        } // signed integer value
         |
        {8'b11111_000       ,
         opcodeUnary[4:0]   , // operation code for unary operations
         value[18:0]        } // integer to parameterize operations

reg [31:0]          acc[0:p-1]      // array's accumuulator
reg                 cr[0:p-1]       // array's carry
reg[$clog2(m)-1:0]  addrReg[0:p-1]  // array's address reg.
reg[n-1:0]          mem[0:m]        // array's data memory
reg[$clog2(p)-1]    act[0:p-1]      // activate counter
reg[n-1:0]          sr[0:p-1]       // serial register

val = {{8{value[23]}},value}    // used as operand
addr = value[$clog2(`m)-1:0]    // used as address

b[i] = (act == 0) ? 1 : 0

coOp = acc
************************************************************/

/************************************************************
 Addressing modes defining the right operand:
************************************************************/
`define	imm 3'b000 // op: val
`define dir 3'b001 // op: mem[addr]
`define rel	3'b010 // op: mem[addrReg+addr]
`define rei 3'b011 // op: mem[addrReg+addr]; addrReg = addrReg+addr
`define cim 3'b100 // op: coOp
`define cdr 3'b101 // op: mem[coOp]
`define	crl 3'b110 // op: mem[addrReg+coOp]
`define cri	3'b111 // op: mem[addrReg+coOp]; addrReg = addrReg+coOp	

// FUNCTIONAL INSTRUCTIONS
`define add     5'b00000 // {cr[i],acc[i]} <=  acc[i] + op
`define addcr   5'b00001 // {cr[i],acc[i]} <=  acc[i] + op + cr
`define sub     5'b00010 // {cr[i],acc[i]} <=  acc[i] - op
`define subcr   5'b00011 // {cr[i],acc[i]} <=  acc[i] - op - cr
`define mult    5'b00100 // {cr[i],acc[i]} <= {cr[i], acc[i] * op}
`define bwand   5'b00101 // {cr[i],acc[i]} <= {cr[i], acc[i] & op}
`define bwor    5'b00110 // {cr[i],acc[i]} <= {cr[i], acc[i] | op}
`define bwxor   5'b00111 // {cr[i],acc[i]} <= {cr[i], acc[i] ^ op}
// DATA MOVE INSTRUCTIONS
`define load    5'b01000 // {cr[i],acc[i]} <= {cr[i], op}
`define store   5'b01001 // aMem[address[i]] <= aAcc
`define insval 	5'b01010 // acc[i] <= {acc[i][23:0], val[7:0]}

`define sendio 	5'b01011 // ioReg[i] <= aMem[address]; ioSet
`define getio  	5'b01100 // aMem[address[i]] <= ioReg[i]; ioRst

// SEARCH INSTRUCTIONS
`define search  5'b01101 // act[i] <= (b[i] & acc[i]=op) ? act[i] : act[i]+1
`define csearch	5'b01110 // act[i] <= (b[i-1] & acc[i]=op) ? 0 : act[i]+1
`define insert  5'b01111 // insert in the first active position in sr

// SELECTS NO-OP INSTRUCTIONS
`define contr   5'b11111 // instruction: instr[23:19]

// CONTROL & GLOBAL INSTRUCTIONS for instr[31:24]= 11111_000
`define allact  5'b00000 // act[i] <= 0
`define where   5'b00001 // act[i] <= (b[i]&cond) ? act[i] : [i]+1
`define elsew   5'b00010 // act[i] <= (act[i]=0)?1:(act[i]=1)?0:act[i]
`define back    5'b00011 // act[i] <= (act[i]=0)? 0 : act[i]-1
`define selsh   5'b00100 // selection global shift right

//`define selsh	5'b00101 //
`define delete  5'b00110 // delete in first position in sr
`define getsr   5'b00111 // acc[i] <= sr[i]
`define sendsr  5'b01000 // sr[i] <= acc[i]

`define ixload  5'b01001 // acc[i] <= i

`define setred 	5'b01010 // set reduce's function

`define sradd   5'b11011 // acc[i] <= acc[i] + sr[i]
`define srmin   5'b11100 // acc[i] <= min(acc[i], sr[i])
`define srmax   5'b11101 // acc[i] <= max(acc[i], sr[i])

`define arela   5'b11110 // aAddrReg <= aAcc
`define shift   5'b11111 // local_shift(val[2:0])